/* intparse.h - Functions to parse integers from strings.
 *
 * Copyright (C) 2001  Oskar Liljeblad
 *
 * This file is part of the file renaming utilities (renameutils)
 * and regex-markup.
 *
 * This software is copyrighted work licensed under the terms of the
 * GNU General Public License. Please consult the file `COPYING' for
 * details.
 */

#include <config.h>
#include "intparse.h"

bool
parse_int8(const char *instr, int8_t *outint)
{
	int8_t value = 0;

	if (*instr == '-') {
		for (instr++; *instr != '\0'; instr++) {
			int8_t c = *instr - '0';
			if (c < 0 || c > 9)
				return FALSE;
			if (value < INT8_MIN/10 || (value == INT8_MIN/10 && c > -(INT8_MIN%10)))
				return FALSE;
			value = value*10 - c;
		}
	} else {
		for (; *instr != '\0'; instr++) {
			int8_t c = *instr - '0';
			if (c < 0 || c > 9)
				return FALSE;
			if (value > INT8_MAX/10 || (value == INT8_MAX/10 && c > INT8_MAX%10))
				return FALSE;
			value = value*10 + c;
		}
	}
	*outint = value;

	return TRUE;
}

bool
parse_int16(const char *instr, int16_t *outint)
{
	int16_t value = 0;

	if (*instr == '-') {
		for (instr++; *instr != '\0'; instr++) {
			int8_t c = *instr - '0';
			if (c < 0 || c > 9)
				return FALSE;
			if (value < INT16_MIN/10 || (value == INT16_MIN/10 && c > -(INT16_MIN%10)))
				return FALSE;
			value = value*10 - c;
		}
	} else {
		for (; *instr != '\0'; instr++) {
			int8_t c = *instr - '0';
			if (c < 0 || c > 9)
				return FALSE;
			if (value > INT16_MAX/10 || (value == INT16_MAX/10 && c > INT16_MAX%10))
				return FALSE;
			value = value*10 + c;
		}
	}
	*outint = value;

	return TRUE;
}

bool
parse_int32(const char *instr, int32_t *outint)
{
	int32_t value = 0;

	if (*instr == '-') {
		if (instr[1] == '\0')		/* FIXME: this should be done on all!!! */
			return FALSE;
		for (instr++; *instr != '\0'; instr++) {
			int8_t c = *instr - '0';
			if (c < 0 || c > 9)
				return FALSE;
			if (value < INT32_MIN/10L || (value == INT32_MIN/10L && c > -(INT32_MIN%10L)))
				return FALSE;
			value = value*10L - c;
		}
	} else {
		if (*instr == '\0')			/* FIXME: this should be done on all!!! */
			return FALSE;
		for (; *instr != '\0'; instr++) {
			int8_t c = *instr - '0';
			if (c < 0 || c > 9)
				return FALSE;
			if (value > INT32_MAX/10L || (value == INT32_MAX/10L && c > INT32_MAX%10L))
				return FALSE;
			value = value*10L + c;
		}
	}
	*outint = value;

	return TRUE;
}

bool
parse_int64(const char *instr, int64_t *outint)
{
	int64_t value = 0;

	if (*instr == '-') {
		for (instr++; *instr != '\0'; instr++) {
			int8_t c = *instr - '0';
			if (c < 0 || c > 9)
				return FALSE;
			if (value < INT64_MIN/10LL || (value == INT64_MIN/10LL && c > -(INT64_MIN%10LL)))
				return FALSE;
			value = value*10LL - c;
		}
	} else {
		for (; *instr != '\0'; instr++) {
			int8_t c = *instr - '0';
			if (c < 0 || c > 9)
				return FALSE;
			if (value > INT64_MAX/10LL || (value == INT64_MAX/10LL && c > INT64_MAX%10LL))
				return FALSE;
			value = value*10LL + c;
		}
	}
	*outint = value;

	return TRUE;
}

bool
parse_uint8(const char *instr, uint8_t *outint)
{
	uint8_t value = 0;

	for (; *instr != '\0'; instr++) {
		int8_t c = *instr - '0';
		if (c < 0 || c > 9)
			return FALSE;
		if (value > UINT8_MAX/10 || (value == UINT8_MAX/10 && c > UINT8_MAX%10))
			return FALSE;
		value = value*10 + c;
	}
	*outint = value;

	return TRUE;
}

bool
parse_uint16(const char *instr, uint16_t *outint)
{
	uint16_t value = 0;

	for (; *instr != '\0'; instr++) {
		int8_t c = *instr - '0';
		if (c < 0 || c > 9)
			return FALSE;
		if (value > UINT16_MAX/10 || (value == UINT16_MAX/10 && c > UINT16_MAX%10))
			return FALSE;
		value = value*10 + c;
	}
	*outint = value;

	return TRUE;
}

bool
parse_uint32(const char *instr, uint32_t *outint)
{
	uint32_t value = 0;

	for (; *instr != '\0'; instr++) {
		int8_t c = *instr - '0';
		if (c < 0 || c > 9)
			return FALSE;
		if (value > UINT32_MAX/10L || (value == UINT32_MAX/10L && c > UINT32_MAX%10))
			return FALSE;
		value = value*10L + c;
	}
	*outint = value;

	return TRUE;
}

bool
parse_uint64(const char *instr, uint64_t *outint)
{
	uint64_t value = 0;

	for (; *instr != '\0'; instr++) {
		int8_t c = *instr - '0';
		if (c < 0 || c > 9)
			return FALSE;
		if (value > UINT64_MAX/10LL || (value == UINT64_MAX/10LL && c > UINT64_MAX%10LL))
			return FALSE;
		value = value*10LL + c;
	}
	*outint = value;

	return TRUE;
}
