/* planactions.c - `apply' and `retry' commands for interactive qmv.
 *
 * Copyright (C) 2001  Oskar Liljeblad
 *
 * This file is part of the file renaming utilities (renameutils).
 *
 * This software is copyrighted work licensed under the terms of the
 * GNU General Public License. Please consult the file `COPYING' for
 * details.
 */

#include <config.h>
#include "common/error.h"
#include "common/memory.h"
#include "qmv.h"

void
apply_command(char **args)
{
	apply_plan(plan);
}

void
retry_command(char **args)
{
	retry_plan(plan);
}

bool
apply_plan(RenamePlan *plan)
{
	Iterator *it;
	bool changed = FALSE;
	bool failed = FALSE;

	if (plan == NULL) {
		printf("no plan - use `list' and `edit'\n");
		return FALSE;
	}
	if (!llist_is_empty(plan->error)) {
		printf("plan contains errors - will only rename OK files\n");
		return FALSE;
	}
	if (llist_is_empty(plan->ok)) {
		printf("plan is empty (no changes made).\n");
		return TRUE;
	}

	if (!cwd_to_renames_directory())
		return FALSE;

	for (it = llist_iterator(plan->ok); iterator_has_next(it); ) {
		FileSpec *spec = iterator_next(it);
		char *name_old;
		char *name_new;
		
		if (spec->renamed == RENAME_FAILED)
			failed = TRUE;
		if (spec->renamed != RENAME_UNDONE)
			continue;
		
		name_old = quote_output_file(spec->old_name);
		name_new = quote_output_file(spec->new_name);
		printf("%s -> %s\n", name_old, name_new);
		free(name_old);
		free(name_new);
		
		changed = TRUE;
		if (!dummy) {
			if (rename(spec->old_name, spec->new_name) < 0) {
				warn_errno("cannot rename %s to %s", spec->old_name, spec->new_name);
				spec->renamed = RENAME_FAILED;
				failed = TRUE;
				printf("renaming failed. `apply' aborted.\n");
				break;
			} else {
				spec->renamed = RENAME_COMPLETE;
			}
		}
	}
	iterator_free(it);

	if (!changed)
		printf("nothing to apply - no changes made.\n");
	if (failed)
		printf("there are failed renames. retry them with `retry'.\n");

	if (!cwd_from_renames_directory())
		return FALSE;

	return !failed;
}

bool
retry_plan(RenamePlan *plan)
{
	Iterator *it;
	bool changed = FALSE;

	if (plan == NULL) {
		printf("no plan - use `list' and `edit'\n");
		return FALSE;
	}
	if (llist_is_empty(plan->ok)) {
		printf("plan is empty (no changes made).\n");
		return TRUE;
	}

	for (it = llist_iterator(plan->ok); iterator_has_next(it); ) {
		FileSpec *spec = iterator_next(it);
		if (spec->renamed != RENAME_FAILED)
			continue;
		
		printf("%s -> %s\n", spec->old_name, spec->new_name);
		changed = TRUE;
		if (!dummy) {
			if (rename(spec->old_name, spec->new_name) < 0) {
				warn_errno("cannot rename %s to %s", spec->old_name, spec->new_name);
				spec->renamed = RENAME_FAILED;
				printf("renaming failed. `retry' aborted.\n");
				break;
			} else {
				spec->renamed = RENAME_COMPLETE;
			}
		}
		changed = TRUE;
	}
	iterator_free(it);

	if (!changed)
		printf("no failed renames - no changes made.\n");

	return TRUE;
}
